var Client = require('../client')
  , util = require(process.binding('natives').util ? 'util' : 'sys')
  , qs = require('querystring');

var HTMLFile = module.exports = function(){
  Client.apply(this, arguments);
};

util.inherits(HTMLFile, Client);
  
HTMLFile.prototype._onConnect = function(req, res){
  var self = this, body = '';
  switch (req.method){
    case 'GET':
      Client.prototype._onConnect.call(this, req, res);
      this.response.useChunkedEncodingByDefault = true;
      this.response.shouldKeepAlive = true;
      this.response.writeHead(200, {
        'Content-Type': 'text/html',
        'Connection': 'keep-alive',
        'Transfer-Encoding': 'chunked'
      });
      this.response.write('<html><body>' + new Array(245).join(' '));
      this._payload();
      break;
      
    case 'POST':
      req.addListener('data', function(message){
        body += message;
      });
      req.addListener('end', function(){
        try {
          var msg = qs.parse(body);
          self._onMessage(msg.data);
        } catch(e){}
        res.writeHead(200, {'Content-Type': 'text/plain'});
        res.write('ok');
        res.end();
      });
      break;
  }
};
  
HTMLFile.prototype._write = function(message){
  if (this._open)
    this.response.write('<script>parent.s._('+ JSON.stringify(message) +', document);</script>'); //json for escaping
};



import time
import threading

import rocket 
from aspen.server import BaseEngine


class Engine(BaseEngine):
    
    rocket_server = None # a rocket.CherryPyWSGIServer instance

    def bind(self):
        self.rocket_server = rocket.CherryPyWSGIServer( self.website.address
                                                      , self.website
                                                       )

    def start(self):
        self.rocket_server.start()

    def stop(self):
        self.rocket_server.stop()

    def start_restarter(self, check_all):

        def loop():
            while True:
                try:
                    check_all()
                except SystemExit:
                    self.rocket_server.stop()
                time.sleep(0.5)

        checker = threading.Thread(target=loop)
        checker.daemon = True
        checker.start()



from aspen.server import BaseEngine
from pants import cycle, engine
from pants.contrib.http import HTTPServer
from pants.contrib.wsgi import WSGIConnector


class Engine(BaseEngine):

    def bind(self):
        connector = WSGIConnector(self.website)
        self.server = HTTPServer(connector)
        self.server.listen( host=self.website.address[0]
                          , port=self.website.address[1]
                           )

    def start(self):
        engine.start()

    def stop(self):
        engine.stop()

    def start_restarter(self, check_all):
        cycle(check_all, 0.5)



import urlparse

from aspen.socket_io import utils


class Client(object):
    """Represent a web browser that wants a socket.
    """

    timeout = 8000
    heartbeatInterval = 10000
    closeTimeout = 0

    def __init__(self, listener, req, res, options, head):
        self.listener = listener
        self.connections = 0
        self._open = False
        self._heartbeats = 0
        self.connected = False
        self.upgradeHead = head
        self._onConnect(req, res)

    def send(self, message):
        if ( not self._open 
           or not (  self.connection.readyState == 'open' 
                  or self.connection.readyState == 'writeOnly' 
                    )):
            return self._queue(message)
        self._write(encode(message))
        return self

    def broadcast(self, message):
        if self.sessionId is None:
            return self
        self.listener.broadcast(message, self.sessionId)
        return self

    def _onMessage(self, data):
        messages = decode(data)
        if (messages == False):
            return self.listener.options.log('Bad message received from client ' + self.sessionId)
        for frame in messages:
            frame = messages[i].substr(0, 3)
            if frame == '~h~':
                return self._onHeartbeat(messages[i].substr(3))
            else:
                assert frame == '~j~' # sanity check
                try:
                    messages[i] = JSON.parse(messages[i].substr(3))
                except:
                    messages[i] = {}
            self.emit('message', messages[i])
            self.listener._onClientMessage(messages[i], self)

    def _onConnect(self, req, res):
        self.request = req
        self.response = res
        self.connection = req.connection

        def on_end():
            self._onClose()
            if (self.connection):
                self.connection.destroy()
        self.connection.addListener('end', on_end)
        
        if (req):
            def on_error_req(err):
                req.end and req.end() or req.destroy and req.destroy()
            req.addListener('error', on_error_req)
            
            if (res):
                def on_error_res(err):
                    res.end and res.end() or res.destroy and res.destroy()
                res.addListener('error', on_error_res)

            def on_error(err):
                req.connection.end and req.connection.end() or req.connection.destroy and req.connection.destroy()
            req.connection.addListener('error', on_error_connection)
                    
            if (self._disconnectTimeout):
                clearTimeout(self._disconnectTimeout)

    def _payload(self):
        payload = []
        
        self.connections += 1
        self.connected = True
        self._open = True
        
        if (not self.handshaked):
            self._generateSessionId()
            payload.push(self.sessionId)
            self.handshaked = True
        
        payload = payload.concat(self._writeQueue or [])
        self._writeQueue = []

        if (payload.length):
            self._write(encode(payload))
        if (self.connections == 1):
            self.listener._onClientConnect(self)
        if (self.options.timeout):
            self._heartbeat()
        
    def _heartbeat(self):
        def heartbeat():
            self._heartbeats += 1
            self.send('~h~' + self._heartbeats)
            def timeout():
                self._onClose()
            self._heartbeatTimeout = setTimeout(timeout, self.timeout)

        self._heartbeatInterval = setTimeout(heartbeat, self.heartbeatInterval)
        
    def _onHeartbeat(self, h):
        if (h == self._heartbeats):
            clearTimeout(self._heartbeatTimeout)
            self._heartbeat()

    def _onClose(self, skipDisconnect):
        if (not self._open):
            return self
        if (self._heartbeatInterval):
            clearTimeout(self._heartbeatInterval)
        if (self._heartbeatTimeout):
            clearTimeout(self._heartbeatTimeout)
        self._open = False
        self.request = null
        self.response = null
        if (not skipDisconnect):
            if self.handshaked:
                self._disconnectTimeout = setTimeout( self._onDisconnect
                                                    , self.closeTimeout
                                                     )
            else:
                self._onDisconnect()

    def _onDisconnect(self):
        if (self._open):
            self._onClose(True)
        if (self._disconnectTimeout):
            clearTimeout(self._disconnectTimeout)
        self._writeQueue = []
        self.connected = False
        if (self.handshaked):
            self.emit('disconnect')
            self.listener._onClientDisconnect(self)
            self.handshaked = False

    def _queue(self, message):
        self._writeQueue = self._writeQueue or []
        self._writeQueue.push(message)
        return self

    def _generateSessionId(self):
        self.sessionId = Math.random().toString().substr(2)
        return self

    def _verifyOrigin(self, origin):
        origins = self.listener.options.origins
        if (origins.indexOf('*:*') != -1):
            return True
        if (origin): 
            try:
                parts = urlparse(origin)
                return (origins.indexOf(parts.host + ':' + parts.port) != -1 or
                        origins.indexOf(parts.host + ':*') != -1 or
                        origins.indexOf('*:' + parts.port) != -1)
            except:
                pass
        return False




import socket
import sys
    
import eventlet
import eventlet.wsgi
from aspen.server import BaseEngine


class DevNull:
    def write(self, msg):
        pass


class Engine(BaseEngine):

    eventlet_socket = None # a socket, per eventlet

    def bind(self):
        self.eventlet_socket = eventlet.listen( self.website.address
                                              , self.website.sockfam
                                               )
        if eventlet.version_info <= (0, 9, 15):
            # Work around https://bitbucket.org/which_linden/eventlet/issue/86/
            if sys.platform[:3] != "win":
                self.eventlet_socket.setsockopt( socket.SOL_SOCKET
                                               , socket.SO_REUSEADDR, 1
                                                )

    def start(self):
        eventlet.wsgi.server(self.eventlet_socket, self.website, log=DevNull())

    def start_restarter(self, check_all):
        def loop():
            while True:
                check_all()
                eventlet.sleep(0.5)
        eventlet.spawn_n(loop)



"""Define configuration objects.
"""
import logging
import mimetypes
import os
import socket
import sys
from os.path import dirname, expanduser, isdir, join, realpath

import aspen
from aspen.configuration.aspenconf import AspenConf 
from aspen.configuration.exceptions import ConfigurationError
from aspen.configuration.hooks import HooksConf
from aspen.configuration.optparser import optparser, validate_address
from aspen._tornado.template import Loader
from aspen.configuration.logging_ import configure_logging
from aspen.configuration.colon import colonize


class Configurable(object):
    """Mixin object for aggregating configuration from several sources.
    """

    def configure(self, argv):
        """Takes an argv list, and gives it straight to optparser.parse_args.

        The argv list should not include the executable name.

        """

        self.__names = [] # keep track of what configuration we configure


        # Parse argv.
        # ===========

        opts, args = optparser.parse_args(argv)


        # Orient ourselves.
        # =================

        self.root = root = find_root(args)
        os.chdir(root)

        self.dotaspen = dotaspen = join(self.root, '.aspen')
        if isdir(dotaspen):
            if sys.path[0] != dotaspen:
                sys.path.insert(0, dotaspen)

        self.__names.extend(['root', 'dotaspen'])


        # Set some attributes.
        # ====================

        self.conf = load_conf(expanduser, dotaspen)
        self.template_loader = Loader(dotaspen)
        self.__names.extend(['conf', 'template_loader'])

        init_mimetypes(mimetypes, dotaspen)
        self.default_mimetype = load_default_mimetype(self.conf)
        self.default_filenames = load_default_filenames(self.conf)
        self.json_content_type = load_json_content_type(self.conf)
        self.show_tracebacks = self.conf.aspen.no('show_tracebacks')
        self.__names.extend(['default_mimetype', 'default_filenames', 
            'json_content_type', 'show_tracebacks'])

        self.hooks = load_hooks(expanduser, dotaspen)
        self.__names.append('hooks')

        self.engine = load_engine(opts, self)
        self.changes_kill = self.conf['aspen.cli'].no('changes_kill')
        self.__names.extend(['engine', 'changes_kill'])

        self.address, self.sockfam = load_address_sockfam(opts, self.conf)
        self.port = load_port(self.address, self.sockfam)
        self.sock = None
        self.__names.extend(['address', 'sockfam', 'port', 'sock'])

        r = configure_logging(opts, dotaspen, self.conf)
        self.log_filename, self.log_filter, self.log_format, self.log_level = r
        self.__names.extend(['log_filename', 'log_filter', 'log_format', 
            'log_level'])

    
    @classmethod
    def from_argv(cls, argv):
        o = cls()
        o.configure(argv)
        return o

    def copy_configuration_to(self, other):
        """Given another object, shallow copy attributes to it.
        """
        for name in self.__names:
            setattr(other, name, getattr(self, name))

def find_root(args):        
    """This can only be passed on the command line.
    """
    if args:
        root = args[0]
    else:
        try:
            # Under supervisord, the following raises 
            #   OSError: [Errno 2] No such file or directory
            # So be sure to pass a directory in on the command line, or cwd
            # using supervisord's own facility for that.
            root = os.getcwd()
        except OSError:
            raise ConfigurationError("Could not get a current working "
                                     "directory. You can specify the site "
                                     "root on the command line.")
    root = realpath(root)
    if not isdir(root):
        msg = "%s does not point to a directory" % root
        raise ConfigurationError(msg)
    return root

def load_conf(expanduser, dotaspen):
    return AspenConf( '/etc/aspen/aspen.conf'
                    , '/usr/local/etc/aspen/aspen.conf'
                    , expanduser('~/.aspen/aspen.conf') 
                    , join(dotaspen, 'aspen.conf')
                     ) # later overrides earlier

def init_mimetypes(mimetypes, dotaspen):
    mimetypes.knownfiles = [ join(dirname(__file__), 'mime.types')
                           , '/etc/mime.types'
                           , '/usr/local/etc/mime.types'
                           , join(dotaspen, 'mime.types')
                            ] # later overrides earlier
    mimetypes.init()

def load_engine(opts, configuration):
    conf = configuration.conf
    if opts.engine is not None:     # use command line if present
        engine_name = opts.engine
    else:                           # fall back to aspen.conf
        engine_name = conf['aspen.cli'].get('engine', aspen.ENGINES[0])
        if engine_name not in aspen.ENGINES:
            msg = "engine is not one of {%s}: %%s" % (','.join(aspen.ENGINES))
            raise ConfigurationError(msg % engine)
    try: 
        exec 'from aspen.server.%s_ import Engine' % engine_name
    except ImportError:
        # ANSI colors: 
        #   http://stackoverflow.com/questions/287871/
        #   http://en.wikipedia.org/wiki/ANSI_escape_code#CSI_codes
        print >> sys.stderr
        print >> sys.stderr, ( "\033[1;31mImportError loading the "
                             + "%s engine:\033[0m" % engine_name
                              )
        print >> sys.stderr
        raise

    engine = Engine(engine_name, configuration)
    return engine

def load_default_mimetype(conf):
    return conf.aspen.get('default_mimetype', 'text/plain')

def load_default_filenames(conf):
    default_filenames = conf.aspen.get('default_filenames', 'index.html')
    default_filenames = default_filenames.split()
    default_filenames = [x.strip(',') for x in default_filenames]
    default_filenames = [x for x in default_filenames if x]
    default_filenames = [x.split(',') for x in default_filenames]
    out = []
    for nested in default_filenames:
        out.extend(nested)
    return out

def load_json_content_type(conf):
    return conf.aspen.get('json_content_type', 'application/json')

def load_hooks(expanduser, dotaspen):
    return HooksConf( join(dirname(__file__), 'hooks.conf')
                    , '/etc/aspen/hooks.conf'
                    , '/usr/local/etc/aspen/hooks.conf'
                    , expanduser('~/.aspen/hooks.conf')
                    , join(dotaspen, 'hooks.conf')
                     ) # later comes after earlier, per section

def load_address_sockfam(opts, conf):
    """These can be set either on the command line or in the conf file.
    """
    if getattr(opts, 'have_address', False):        # first check CLI
        address = opts.address
        sockfam = opts.sockfam
    elif 'address' in conf['aspen.cli']:            # then check conf
        address, sockfam = validate_address(conf['aspen.cli']['address'])
    else:                                           # default from optparse
        address = opts.address
        sockfam = socket.AF_INET

    return address, sockfam

def load_port(address, sockfam):
    if sockfam == socket.AF_INET:
        port = address[1]
    else:
        port = None
    return port




import time
import threading

from aspen._cherrypy.wsgiserver import CherryPyWSGIServer
from aspen.server import BaseEngine


class Engine(BaseEngine):

    cp_server = None # a CherryPyWSGIServer instance

    def bind(self):
        self.cp_server = CherryPyWSGIServer(self.website.address, self.website)

    def start(self):
        self.cp_server.start()

    def stop(self):
        self.cp_server.stop()

    def start_restarter(self, check_all):

        def loop():
            while True:
                try:
                    check_all()
                except SystemExit:
                    self.cp_server.interrupt = SystemExit
                time.sleep(0.5)

        checker = threading.Thread(target=loop)
        checker.daemon = True
        checker.start()



"""Module for loading objects specified in colon notation.
"""
import string
from os.path import basename

from aspen.configuration.exceptions import ConfFileError


INITIAL = '_' + string.letters
INNER = INITIAL + string.digits
def is_valid_identifier(s):
    """Given a string of length > 0, return a boolean.

        >>> is_valid_identifier('.svn')
        False
        >>> is_valid_identifier('svn')
        True
        >>> is_valid_identifier('_svn')
        True
        >>> is_valid_identifier('__svn')
        True
        >>> is_valid_identifier('123')
        False
        >>> is_valid_identifier('r123')
        True

    """
    try:
        assert s[0] in INITIAL
        assert False not in [x in INNER for x in s]
        return True
    except AssertionError:
        return False


class ColonizeError(ConfFileError):
    pass

class ColonizeBadColonsError(ColonizeError): pass
class ColonizeBadObjectError(ColonizeError): pass
class ColonizeBadModuleError(ColonizeError): pass


def colonize(name, filename='n/a', lineno=-1):
    """Given a name in colon notation and some error helpers, return an object.

    The format of name is a subset of setuptools entry_point format: a
    dotted module name, followed by a colon and a dotted identifier naming
    an object within the module.

    """

    if name.count(':') != 1:
        msg = "'%s' is not valid colon notation" % name
        raise ColonizeBadColonsError(msg, filename, lineno)

    modname, objname = name.rsplit(':', 1) # no rsplit < Python 2.4

    for _name in modname.split('.'):
        if not is_valid_identifier(_name):
            msg = ( "'%s' is not valid colon notation: " % name
                  + "bad module name '%s'" % _name
                   )
            raise ColonizeBadModuleError(msg, filename, lineno)

    try:
        if '.' in modname:
            pkg, mod = modname.rsplit('.', 1)
            exec 'from %s import %s as obj' % (pkg, mod)
        else:
            exec 'import %s as obj' % modname
    except ImportError, err:
        newmsg = "%s [%s, line %s]" % (err.args[0], basename(filename), lineno)
        err.args = (newmsg,)
        raise # preserve the original traceback

    for _name in objname.split('.'):
        if not is_valid_identifier(_name):
            msg = ( "'%s' is not valid colon notation: " % name
                  + "bad object name '%s'" % _name
                   )
            raise ColonizeBadObjectError(msg, filename, lineno)
        try:
            obj = getattr(obj, _name)
        except AttributeError, err:
            newmsg = "%s [%s, line %s]" % ( err.args[0]
                                          , basename(filename)
                                          , lineno
                                           )
            err.args = (newmsg,)
            raise # preserve the original traceback

    return obj



var Client = require('../client')
  , Stream = require('net').Stream
  , EventEmitter = require('events').EventEmitter
  , url = require('url')
  , util = require(process.binding('natives').util ? 'util' : 'sys')
  , crypto = require('crypto');

WebSocket = module.exports = function(){
  Client.apply(this, arguments);
};

util.inherits(WebSocket, Client);

WebSocket.prototype._onConnect = function(req, socket){
  var self = this
    , headers = [];
  
  if (!req.connection.setTimeout){
    req.connection.end();
    return false;
  }

  this.parser = new Parser();
  this.parser.on('data', self._onMessage.bind(this));
  this.parser.on('error', self._onClose.bind(this));

  Client.prototype._onConnect.call(this, req);
    
  if (this.request.headers.upgrade !== 'WebSocket' || !this._verifyOrigin(this.request.headers.origin)){
    this.listener.options.log('WebSocket connection invalid or Origin not verified');
    this._onClose();
    return false;
  }
  
  var origin = this.request.headers.origin,
      location = (origin && origin.substr(0, 5) == 'https' ? 'wss' : 'ws')
               + '://' + this.request.headers.host + this.request.url;
  
  this.waitingForNonce = false;
  if ('sec-websocket-key1' in this.request.headers){
    /*  We need to send the 101 response immediately when using Draft 76 with
      	a load balancing proxy, such as HAProxy.  In order to protect an
      	unsuspecting non-websocket HTTP server, HAProxy will not send the
      	8-byte nonce through the connection until the Upgrade: WebSocket
      	request has been confirmed by the WebSocket server by a 101 response
      	indicating that the server can handle the upgraded protocol.  We
      	therefore must send the 101 response immediately, and then wait for
      	the nonce to be forwarded to us afterward in order to finish the
      	Draft 76 handshake.
      */
    
    // If we don't have the nonce yet, wait for it.
    if (!(this.upgradeHead && this.upgradeHead.length >= 8)) {
      this.waitingForNonce = true;
    }
    
    headers = [
      'HTTP/1.1 101 WebSocket Protocol Handshake',
      'Upgrade: WebSocket',
      'Connection: Upgrade',
      'Sec-WebSocket-Origin: ' + origin,
      'Sec-WebSocket-Location: ' + location
    ];
    
    if ('sec-websocket-protocol' in this.request.headers){
      headers.push('Sec-WebSocket-Protocol: ' + this.request.headers['sec-websocket-protocol']);
    }
  } else {
    headers = [
      'HTTP/1.1 101 Web Socket Protocol Handshake',
      'Upgrade: WebSocket',
      'Connection: Upgrade',
      'WebSocket-Origin: ' + origin,
      'WebSocket-Location: ' + location
    ];
    
  }

  try {
    this.connection.write(headers.concat('', '').join('\r\n'));
  } catch(e){
    this._onClose();
  }
  
  this.connection.setTimeout(0);
  this.connection.setNoDelay(true);
  this.connection.setEncoding('utf-8');
  
  if (this.waitingForNonce) {
  	// Since we will be receiving the binary nonce through the normal HTTP
  	// data event, set the connection to 'binary' temporarily
  	this.connection.setEncoding('binary');
  	this._headers = headers;
  }
  else {
  	if (this._proveReception(headers)) this._payload();
  }
  
  this.buffer = "";
  
  this.connection.addListener('data', function(data){
    self.buffer += data;
    if (self.waitingForNonce) {
  		if (self.buffer.length < 8) { return; }
  		// Restore the connection to utf8 encoding after receiving the nonce
  		self.connection.setEncoding('utf8');
  		self.waitingForNonce = false;
  		// Stuff the nonce into the location where it's expected to be
  		self.upgradeHead = self.buffer.substr(0,8);
  		self.buffer = self.buffer.substr(8);
  		if (self.buffer.length > 0) {
  		  self.parser.add(self.buffer);
  		}
  		if (self._proveReception(self._headers)) { self._payload(); }
  		return;
  	}
  	
    self.parser.add(data);
  });

};

// http://www.whatwg.org/specs/web-apps/current-work/complete/network.html#opening-handshake
WebSocket.prototype._proveReception = function(headers){
  var self = this
    , k1 = this.request.headers['sec-websocket-key1']
    , k2 = this.request.headers['sec-websocket-key2'];
  
  if (k1 && k2){
    var md5 = crypto.createHash('md5');

    [k1, k2].forEach(function(k){
      var n = parseInt(k.replace(/[^\d]/g, '')),
          spaces = k.replace(/[^ ]/g, '').length;
          
      if (spaces === 0 || n % spaces !== 0){
        self.listener.options.log('Invalid WebSocket key: "' + k + '". Dropping connection');
        self._onClose();
        return false;
      }

      n /= spaces;
      
      md5.update(String.fromCharCode(
        n >> 24 & 0xFF,
        n >> 16 & 0xFF,
        n >> 8  & 0xFF,
        n       & 0xFF));
    });

    md5.update(this.upgradeHead.toString('binary'));
    
    try {
      this.connection.write(md5.digest('binary'), 'binary');
    } catch(e){
      this._onClose();
    }
  }
  
  return true;
};

WebSocket.prototype._write = function(message){
  try {
    this.connection.write('\u0000', 'binary');
    this.connection.write(message, 'utf8');
    this.connection.write('\uffff', 'binary');
  } catch(e){
    this._onClose();
  }
};

WebSocket.httpUpgrade = true;

function Parser(){
  this.buffer = '';
  this.i = 0;
};

Parser.prototype.__proto__ = EventEmitter.prototype;

Parser.prototype.add = function(data){
  this.buffer += data;
  this.parse();
};

Parser.prototype.parse = function(){
  for (var i = this.i, chr, l = this.buffer.length; i < l; i++){
    chr = this.buffer[i];
    if (i === 0){
      if (chr != '\u0000')
        this.error('Bad framing. Expected null byte as first frame');
      else
        continue;
    }
    if (chr == '\ufffd'){
      this.emit('data', this.buffer.substr(1, this.buffer.length - 2));
      this.buffer = this.buffer.substr(i + 1);
      this.i = 0;
      return this.parse();
    }
  }
};

Parser.prototype.error = function(reason){
  this.buffer = '';
  this.i = 0;
  this.emit('error', reason);
  return this;
};



import os
import sys
import time
from aspen.http.response import Response


# Find a json module.
# ===================
# The standard library includes simplejson as json since 2.6, but without the 
# C speedups. So we prefer simplejson if it is available.

try:
    import simplejson as json
except ImportError:
    try:
        import json
    except ImportError:
        json = None


__version__ = "~~VERSION~~"
WINDOWS = sys.platform[:3] == 'win'
ENGINES = ['cherrypy', 'diesel', 'eventlet', 'pants', 'rocket']


def thrash():
    """This is a very simple tool to restart a process when it dies.

    It's designed to restart aspen in development when it dies because files
    have changed and you set changes_kill to 'yes' in the [aspen.cli] section
    of aspen.conf.

    http://aspen.io/thrash/

    """
    try:
        while 1:
            os.system(' '.join(sys.argv[1:]))
            time.sleep(1)
    except KeyboardInterrupt:
        pass



import hashlib
import logging
import time
import traceback
from struct import pack

try:
    import json
except ImportError:
    import simplejson as json

from aspen import simplates
from aspen.http import Response
from aspen.socket_io.transports import xhr_polling
from aspen.socket_io.utils import encode
from diesel import ConnectionClosed, first, fork, receive, send, until
from diesel.util.queue import Queue, QueueTimeout


log = logging.getLogger('aspen.socket_io')


def xhr_polling_handler(request, inq, outq):
    while True:
        try:
            xhr_request = inq.get(timeout=0.5)
        except QueueTimeout:
            pass
        else:
            log.info("handling xhr long-polling request")
            log.info(ws_request)
            try:
                simplates.handle(request)
            except Response, response:
                pass
            except:
                response = Response(500, traceback.format_exc())
            outq.put(response)

def handle(request):
    """Handle a request for a websocket.
    """
    if request.transport != 'xhr-polling':
        raise Response(404)

    org = request.headers.one('Origin')
    inq = Queue()
    outq = Queue()

    def wrap(request, inq, outq):
        handler(request, inq, outq)
        outq.put(WebSocketDisconnect())
    fork(wrap, request, inq, outq)

    while True:
        try:
            log.debug("trying websocket thing")
            typ, val = first(receive=1, waits=[outq.wait_id])
            log.debug(typ)
            log.debug(val)
            if typ == 'receive':
                assert val == '\x00'
                val = until('\xff')[:-1]
                if val == '':
                    inq.put(WebSocketDisconnect())
                else:
                    inq.put(request)
            else:
                try:
                    v = outq.get(waiting=False)
                except QueueEmpty:
                    pass
                else:
                    if type(v) is WebSocketDisconnect:
                        send('\x00\xff')
                        break
                    else:
                        send('\x00%s\xff' % response.to_http(request.version))

        except ConnectionClosed:
            inq.put(WebSocketDisconnect())
            raise ConnectionClosed("remote disconnected")




"""Simplates

Problems with tornado.template:

    - no option to fail silently
    - Loader cache doesn't account for modtime
    - Is this a bug?

        {{ foo }}
        {% for foo in [1,2,3] %}
        {% end %}

    - no loop counters, eh? must do it manually with {% set %}
    - can't do this:

        {% if ... %}
            {% extends %}
        {% else %}
            {% extends %}
        {% end %}

"""
import datetime
import logging
import mimetypes
import os
import stat
import sys
import traceback
from os.path import join

from aspen import json, Response
from _tornado.template import Loader, Template


PAGE_BREAK = chr(12)
log = logging.getLogger('aspen.simplate')


class LoadError(StandardError):
    """Represent a problem parsing a simplate.
    """

class FriendlyEncoder(json.JSONEncoder):
    """Add support for additional types to the default JSON encoder.
    """

    def default(self, obj):
        if isinstance(obj, complex):
            # http://docs.python.org/library/json.html
            out = [obj.real, obj.imag]
        elif isinstance(obj, datetime.datetime):
            # http://stackoverflow.com/questions/455580/
            out = obj.isoformat()
        else:
            out = json.JSONEncoder.default(self, obj)
        return out


# Cache helpers
# =============

__cache = dict()        # cache, keyed to filesystem path

class Entry:
    """An entry in the global simplate cache.
    """

    fspath = ''         # The filesystem path [string]
    modtime = None      # The timestamp of the last change [int]
    quadruple = None    # A post-processed version of the data [4-tuple]
    exc = None          # Any exception in reading or compilation [Exception]

    def __init__(self):
        self.fspath = ''
        self.modtime = 0
        self.quadruple = ()


# Core loader
# ===========

def load_uncached(request):
    """Given a Request object, return three objects (uncached).

    A simplate is a template with two optional Python components at the head of
    the file, delimited by '^L'. The first Python page is exec'd when the
    simplate is first called, and the namespace it populates is saved for all
    subsequent runs. The second Python page is exec'd within the template
    namespace each time the template is rendered.

    If the mimetype does not start with 'text/', then it is only a simplate if
    it has at least one form feed in it. Binary files generally can't be
    decoded using UTF-8. If Python's mimetypes module doesn't know about a
    certain extension, then we default to a configurable value (default is
    text/plain).

    """

    simplate = open(request.fs).read()
    
    # We work with simplates exclusively as a bytestring. Any unicode objects
    # passed in by the user as {{ expressions }} will be encoded with UTF-8 by
    # Tornado.
    
    mimetype = mimetypes.guess_type(request.fs, strict=False)[0]
    if mimetype is None:
        mimetype = request.default_mimetype
  

    # Try to exit early.
    # ==================
    # For non-simplates we want to return None for the first two pages, to
    # avoid those execs during request handling (this is an optimization).

    is_simplate = True # guilty until proven innocent

    if mimetype.startswith('text/') or mimetype == 'application/json':

        # For text formats we can perform a highly accurate test for
        # simplitude.

        c = lambda s: s in simplate
        is_simplate = c("") or c("^L") or c("{%") or c("{{")

    else:

        # For binary formats we must rely on a less-accurate test. This is
        # because a binary file can have s in it without being a simplate--
        # and I've actually seen, in the wild, a file with exactly two s. So
        # we sniff the first few bytes.

        s = lambda s: simplate.startswith(s)
        is_simplate = s('"""') or s('import') or s('from')
           

    if not is_simplate:
        # Static files have no Python pages.
        return (mimetype, None, None, simplate) 
    

    # Parse as a simplate.
    # ====================

    simplate = simplate.replace("^L", PAGE_BREAK)
    npage_breaks = simplate.count(PAGE_BREAK)

    if mimetype == 'application/json': # see: http://aspen.io/simplates/json/
        if json is None:
            raise LoadError("Neither json nor simplejson was found. Try "
                            "installing simplejson to use dynamic JSON "
                            "simplates. See "
                            "http://aspen.io/simplates/json/#libraries for "
                            "more information.")

        template = None
        if npage_breaks == 0:
            imports = ""
            script = simplate
        elif npage_breaks == 1:
            imports, script = simplate.split(PAGE_BREAK)
        else:
            raise SyntaxError( "JSON simplate %s may have at " % request.fs
                             + "most one page breaks; it has "
                             + "%d." % npage_breaks
                              )
    else:
        if npage_breaks == 0:
            script = imports = ""
            template = simplate
        elif npage_breaks == 1:
            imports = ""
            script, template = simplate.split(PAGE_BREAK)
        elif npage_breaks == 2:
            imports, script, template = simplate.split(PAGE_BREAK)
        else:
            raise SyntaxError( "Simplate %s may have at most two " % request.fs
                             + "page breaks; it has %d." % npage_breaks
                              )


    # Standardize newlines.
    # =====================
    # compile requires \n, and doing it now makes the next line easier.

    imports = imports.replace('\r\n', '\n')
    script = script.replace('\r\n', '\n')


    # Pad the beginning of the second page.
    # =====================================
    # This is so we get accurate tracebacks. We used to do this for the
    # template page too, but Tornado templates have some weird error handling
    # that we haven't exposed yet.

    script = ''.join(['\n' for n in range(imports.count('\n'))]) + script


    # Prep our cachable objects and return.
    # =====================================

    namespace = dict()
    namespace['__file__'] = request.fs
    namespace['website'] = request.website
    script = compile(script, request.fs, 'exec')
    if template is not None and template.strip():
        template = Template( template
                           , name = request.fs
                           , loader = request.website.template_loader
                           , compress_whitespace = False
                            )
    else:
        template = None

    exec compile(imports, request.fs, 'exec') in namespace

    return (mimetype, namespace, script, template)


# Cache wrapper
# =============

def load(request):
    """Given a Request object, return four objects (with caching).
    """

    # Get a cache Entry object.
    # =========================

    if request.fs not in __cache:
        entry = Entry()
        #entry.fspath = request.fs # TODO: What was this for?
        __cache[request.fs] = entry

    entry = __cache[request.fs]


    # Process the simplate.
    # =====================

    modtime = os.stat(request.fs)[stat.ST_MTIME]
    if entry.modtime == modtime:                            # cache hit
        if entry.exc is not None:
            raise entry.exc
    else:                                                   # cache miss
        try:
            entry.quadruple = load_uncached(request)
        except Exception, exception:
            # NB: Old-style string exceptions will still raise.
            entry.exc = ( LoadError(traceback.format_exc())
                        , sys.exc_info()[2]
                         )
        else: # reset any previous exception
            entry.exc = None 

        entry.modtime = modtime
        if entry.exc is not None:
            raise entry.exc[0] # TODO Why [0] here, and not above?


    # Return
    # ======
    # Avoid mutating the cached namespace dictionary.

    mimetype, namespace, script, template = entry.quadruple
    if namespace is not None:
        namespace = namespace.copy()
    return (mimetype, namespace, script, template)


# Main callable.
# ==============

def handle(request, response=None):
    """Given a Request, return or raise a Response.
    """
    if response is None:
        response = Response()

    mimetype, namespace, script, template = load(request)

    if namespace is None:
        response.body = template
    else:
       
        # Populate namespace.
        # ===================
    
        namespace.update(request.namespace)
        namespace['request'] = request
        namespace['response'] = response
   

        # Exec the script.
        # ================
    
        if script:
            exec script in namespace
            response = namespace['response']


        # Process the template.
        # =====================
        # If template is None that means that that page was empty.
    
        if template is not None:
            response.body = template.generate(**namespace)

    
    # Set the mimetype.
    # =================
    # We guess based on the filesystem path, not the URL path. Also, we 
    # special case JSON.
   
    if mimetype == 'application/json':
        if template is None:                # dynamic
            if not isinstance(response.body, basestring):
                # json.dumps is guaranteed to exist here.
                response.body = json.dumps( response.body
                                          , cls=FriendlyEncoder
                                           )
        else:                               # static
            pass
        response.headers.set('Content-Type', request.json_content_type)
    
    if response.headers.one('Content-Type') is None:
        if mimetype.startswith('text/'):
            mimetype += "; charset=UTF-8" 
        response.headers.set('Content-Type', mimetype)


    # Send it on back up the line.
    # ============================

    return response
